@@ -1,27 +1,33 @@
 """Module for the Accessory classes."""
 import itertools
 import logging
-
+from typing import TYPE_CHECKING, Any, Callable, Dict, Iterable, List, Optional
 from uuid import UUID
-from pyhap import SUPPORT_QR_CODE, util
-from pyhap.const import (
+
+from . import SUPPORT_QR_CODE, util
+from .const import (
     CATEGORY_BRIDGE,
     CATEGORY_OTHER,
+    HAP_PROTOCOL_VERSION,
     HAP_REPR_AID,
     HAP_REPR_IID,
-    HAP_PROTOCOL_VERSION,
     HAP_REPR_SERVICES,
     HAP_REPR_VALUE,
     STANDALONE_AID,
 )
-from pyhap.iid_manager import IIDManager
-from pyhap.service import Service
+from .iid_manager import IIDManager
+from .service import Service
 
 if SUPPORT_QR_CODE:
     import base36
     from pyqrcode import QRCode
 
 
+if TYPE_CHECKING:
+    from .accessory_driver import AccessoryDriver
+    from .characteristic import Characteristic
+
+
 HAP_PROTOCOL_INFORMATION_SERVICE_UUID = UUID("000000A2-0000-1000-8000-0026BB765291")
 
 logger = logging.getLogger(__name__)
@@ -35,7 +41,13 @@ class Accessory:
 
     category = CATEGORY_OTHER
 
-    def __init__(self, driver, display_name, aid=None):
+    def __init__(
+        self,
+        driver: "AccessoryDriver",
+        display_name: Optional[str],
+        aid: Optional[int] = None,
+        iid_manager: Optional[IIDManager] = None,
+    ) -> None:
         """Initialise with the given properties.
 
         :param display_name: Name to be displayed in the Home app.
@@ -47,24 +59,24 @@ class Accessory:
             will assign the standalone AID to this `Accessory`.
         :type aid: int
         """
-        self.aid = aid
-        self.display_name = display_name
+        self.aid: Optional[int] = aid
+        self.display_name: Optional[str] = display_name
         self.driver = driver
-        self.services = []
-        self.iid_manager = IIDManager()
-        self.setter_callback = None
+        self.services: List[Service] = []
+        self.iid_manager = iid_manager or IIDManager()
+        self.setter_callback: Optional[Callable[[Any], None]] = None
 
         self.add_info_service()
         if aid == STANDALONE_AID:
             self.add_protocol_version_service()
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Return the representation of the accessory."""
         services = [s.display_name for s in self.services]
         return f"<accessory display_name='{self.display_name}' services={services}>"
 
     @property
-    def available(self):
+    def available(self) -> bool:
         """Accessory is available.
 
         If available is False, get_characteristics will return
@@ -75,7 +87,7 @@ class Accessory:
         """
         return True
 
-    def add_info_service(self):
+    def add_info_service(self) -> None:
         """Helper method to add the required `AccessoryInformation` service.
 
         Called in `__init__` to be sure that it is the first service added.
@@ -116,9 +128,16 @@ class Accessory:
                     self.display_name,
                 )
 
-    def add_preload_service(self, service, chars=None):
+    def add_preload_service(
+        self,
+        service: Service,
+        chars: Optional[Iterable["Characteristic"]] = None,
+        unique_id: Optional[str] = None,
+    ) -> Service:
         """Create a service with the given name and add it to this acc."""
         service = self.driver.loader.get_service(service)
+        if unique_id is not None:
+            service.unique_id = unique_id
         if chars:
             chars = chars if isinstance(chars, list) else [chars]
             for char_name in chars:
@@ -127,12 +146,12 @@ class Accessory:
         self.add_service(service)
         return service
 
-    def set_primary_service(self, primary_service):
+    def set_primary_service(self, primary_service: Service) -> None:
         """Set the primary service of the acc."""
         for service in self.services:
             service.is_primary_service = service.type_id == primary_service.type_id
 
-    def add_service(self, *servs):
+    def add_service(self, *servs: Service) -> None:
         """Add the given services to this Accessory.
 
         This also assigns unique IIDS to the services and their Characteristics.
@@ -144,14 +163,14 @@ class Accessory:
         :type: Service
         """
         for s in servs:
+            s.broker = self
             self.services.append(s)
             self.iid_manager.assign(s)
-            s.broker = self
             for c in s.characteristics:
-                self.iid_manager.assign(c)
                 c.broker = self
+                self.iid_manager.assign(c)
 
-    def get_service(self, name):
+    def get_service(self, name: str) -> Optional[Service]:
         """Return a Service with the given name.
 
         A single Service is returned even if more than one Service with the same name
@@ -166,7 +185,7 @@ class Accessory:
         """
         return next((s for s in self.services if s.display_name == name), None)
 
-    def xhm_uri(self):
+    def xhm_uri(self) -> str:
         """Generates the X-HM:// uri (Setup Code URI)
 
         :rtype: str
@@ -193,7 +212,7 @@ class Accessory:
 
         return "X-HM://" + encoded_payload + self.driver.state.setup_id
 
-    def get_characteristic(self, aid, iid):
+    def get_characteristic(self, aid: int, iid: int) -> Optional["Characteristic"]:
         """Get the characteristic for the given IID.
 
         The AID is used to verify if the search is in the correct accessory.
@@ -203,7 +222,7 @@ class Accessory:
 
         return self.iid_manager.get_obj(iid)
 
-    def to_HAP(self):
+    def to_HAP(self, include_value: bool = True) -> Dict[str, Any]:
         """A HAP representation of this Accessory.
 
         :return: A HAP representation of this accessory. For example:
@@ -222,7 +241,9 @@ class Accessory:
         """
         return {
             HAP_REPR_AID: self.aid,
-            HAP_REPR_SERVICES: [s.to_HAP() for s in self.services],
+            HAP_REPR_SERVICES: [
+                s.to_HAP(include_value=include_value) for s in self.services
+            ],
         }
 
     def setup_message(self):
@@ -245,7 +266,7 @@ class Accessory:
             )
         else:
             print(
-                "To use the QR Code feature, use 'pip install " "HAP-python[QRCode]'",
+                "To use the QR Code feature, use 'pip install HAP-python[QRCode]'",
                 flush=True,
             )
             print(
@@ -323,11 +344,18 @@ class Bridge(Accessory):
 
     category = CATEGORY_BRIDGE
 
-    def __init__(self, driver, display_name):
-        super().__init__(driver, display_name, aid=STANDALONE_AID)
+    def __init__(
+        self,
+        driver: "AccessoryDriver",
+        display_name: Optional[str],
+        iid_manager: Optional[IIDManager] = None,
+    ) -> None:
+        super().__init__(
+            driver, display_name, aid=STANDALONE_AID, iid_manager=iid_manager
+        )
         self.accessories = {}  # aid: acc
 
-    def add_accessory(self, acc):
+    def add_accessory(self, acc: "Accessory") -> None:
         """Add the given ``Accessory`` to this ``Bridge``.
 
         Every ``Accessory`` in a ``Bridge`` must have an AID and this AID must be
@@ -360,14 +388,17 @@ class Bridge(Accessory):
 
         self.accessories[acc.aid] = acc
 
-    def to_HAP(self):
+    def to_HAP(self, include_value: bool = True) -> List[Dict[str, Any]]:
         """Returns a HAP representation of itself and all contained accessories.
 
         .. seealso:: Accessory.to_HAP
         """
-        return [acc.to_HAP() for acc in (super(), *self.accessories.values())]
+        return [
+            acc.to_HAP(include_value=include_value)
+            for acc in (super(), *self.accessories.values())
+        ]
 
-    def get_characteristic(self, aid, iid):
+    def get_characteristic(self, aid: int, iid: int) -> Optional["Characteristic"]:
         """.. seealso:: Accessory.to_HAP"""
         if self.aid == aid:
             return self.iid_manager.get_obj(iid)
@@ -378,17 +409,17 @@ class Bridge(Accessory):
 
         return acc.get_characteristic(aid, iid)
 
-    async def run(self):
+    async def run(self) -> None:
         """Schedule tasks for each of the accessories' run method."""
         for acc in self.accessories.values():
             self.driver.async_add_job(acc.run)
 
-    async def stop(self):
+    async def stop(self) -> None:
         """Calls stop() on all contained accessories."""
         await self.driver.async_add_job(super().stop)
         for acc in self.accessories.values():
             await self.driver.async_add_job(acc.stop)
 
 
-def get_topic(aid, iid):
+def get_topic(aid: int, iid: int) -> str:
     return str(aid) + "." + str(iid)
